/*
Copyright 2019 The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
// Code generated by client-gen. DO NOT EDIT.

package v1alpha3

import (
	v1alpha3 "github.com/kubeflow/katib/pkg/apis/controller/suggestions/v1alpha3"
	scheme "github.com/kubeflow/katib/pkg/client/controller/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// SuggestionsGetter has a method to return a SuggestionInterface.
// A group's client should implement this interface.
type SuggestionsGetter interface {
	Suggestions(namespace string) SuggestionInterface
}

// SuggestionInterface has methods to work with Suggestion resources.
type SuggestionInterface interface {
	Create(*v1alpha3.Suggestion) (*v1alpha3.Suggestion, error)
	Update(*v1alpha3.Suggestion) (*v1alpha3.Suggestion, error)
	UpdateStatus(*v1alpha3.Suggestion) (*v1alpha3.Suggestion, error)
	Delete(name string, options *v1.DeleteOptions) error
	DeleteCollection(options *v1.DeleteOptions, listOptions v1.ListOptions) error
	Get(name string, options v1.GetOptions) (*v1alpha3.Suggestion, error)
	List(opts v1.ListOptions) (*v1alpha3.SuggestionList, error)
	Watch(opts v1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1alpha3.Suggestion, err error)
	SuggestionExpansion
}

// suggestions implements SuggestionInterface
type suggestions struct {
	client rest.Interface
	ns     string
}

// newSuggestions returns a Suggestions
func newSuggestions(c *SuggestionV1alpha3Client, namespace string) *suggestions {
	return &suggestions{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the suggestion, and returns the corresponding suggestion object, and an error if there is any.
func (c *suggestions) Get(name string, options v1.GetOptions) (result *v1alpha3.Suggestion, err error) {
	result = &v1alpha3.Suggestion{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("suggestions").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of Suggestions that match those selectors.
func (c *suggestions) List(opts v1.ListOptions) (result *v1alpha3.SuggestionList, err error) {
	result = &v1alpha3.SuggestionList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("suggestions").
		VersionedParams(&opts, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested suggestions.
func (c *suggestions) Watch(opts v1.ListOptions) (watch.Interface, error) {
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("suggestions").
		VersionedParams(&opts, scheme.ParameterCodec).
		Watch()
}

// Create takes the representation of a suggestion and creates it.  Returns the server's representation of the suggestion, and an error, if there is any.
func (c *suggestions) Create(suggestion *v1alpha3.Suggestion) (result *v1alpha3.Suggestion, err error) {
	result = &v1alpha3.Suggestion{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("suggestions").
		Body(suggestion).
		Do().
		Into(result)
	return
}

// Update takes the representation of a suggestion and updates it. Returns the server's representation of the suggestion, and an error, if there is any.
func (c *suggestions) Update(suggestion *v1alpha3.Suggestion) (result *v1alpha3.Suggestion, err error) {
	result = &v1alpha3.Suggestion{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("suggestions").
		Name(suggestion.Name).
		Body(suggestion).
		Do().
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().

func (c *suggestions) UpdateStatus(suggestion *v1alpha3.Suggestion) (result *v1alpha3.Suggestion, err error) {
	result = &v1alpha3.Suggestion{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("suggestions").
		Name(suggestion.Name).
		SubResource("status").
		Body(suggestion).
		Do().
		Into(result)
	return
}

// Delete takes name of the suggestion and deletes it. Returns an error if one occurs.
func (c *suggestions) Delete(name string, options *v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("suggestions").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *suggestions) DeleteCollection(options *v1.DeleteOptions, listOptions v1.ListOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("suggestions").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched suggestion.
func (c *suggestions) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1alpha3.Suggestion, err error) {
	result = &v1alpha3.Suggestion{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("suggestions").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
